import { TransformNode } from "@babylonjs/core/Meshes/transformNode";
import { GLTF2Export } from "@babylonjs/serializers/glTF/2.0/glTFSerializer";
//activate necessary gltf extensions
import "@babylonjs/serializers/glTF/2.0/Extensions/KHR_texture_transform";
import "@babylonjs/serializers/glTF/2.0/Extensions/KHR_materials_unlit";
import { ERR_FAILED, newError } from "../../../utils/error";
import { eq } from "../../../utils/math";
import { Texture } from "../Texture";
/**
 * @deprecated
 */
export class GltfScene {
    /**
     * Experimental GLTF export.
     * @deprecated
     */
    static async Export(pModel, pSceneElements = null, pScaleFactor) {
        //collect nodes to export
        const tExportNodes = new Set();
        const tElements = pSceneElements !== null && pSceneElements !== void 0 ? pSceneElements : pModel.elements;
        for (const tElement of tElements) {
            const tNodes = tElement.getNodes(true);
            for (const tNode of tNodes) {
                tExportNodes.add(tNode);
            }
        }
        //add root scaling node
        let tScalingNode;
        if (pScaleFactor != null) {
            tScalingNode = new TransformNode("gltf_scalenode", pModel.scene);
            tScalingNode.scaling.set(pScaleFactor, pScaleFactor, pScaleFactor);
            for (const tElement of tElements) {
                tElement.node.parent = tScalingNode;
                tElement.updateTransformation();
            }
        }
        //patch textures
        const tPatchedTextures = new Array();
        for (const tTexture of pModel.scene.textures) {
            if (tTexture instanceof Texture) {
                tPatchedTextures.push({
                    tex: tTexture,
                    orgName: tTexture.name,
                    orgOffsetU: tTexture.uOffset,
                    orgOffsetV: tTexture.vOffset
                });
                //assign unique name
                tTexture.name = tTexture.name + "_" + tTexture.uniqueId;
                //set rotationCenter to 0 for unrotated textures to enable KHR_texture_transform
                if (eq(tTexture.wAng, 0) ||
                    (eq(tTexture.uOffset, 0) && eq(tTexture.vOffset, 0) &&
                        eq(tTexture.uScale, 1) && eq(tTexture.vScale, 1))) {
                    //this enables KHR_texture_transform
                    tTexture.wRotationCenter = 0;
                    tTexture.uRotationCenter = 0;
                    tTexture.vRotationCenter = 0;
                    //multiple offset by scale
                    tTexture.uOffset *= tTexture.uScale;
                    tTexture.vOffset *= tTexture.vScale;
                }
            }
        }
        //run gltf export
        const tGltfOptions = {
            shouldExportNode: (pNode) => tExportNodes.has(pNode) || pNode == tScalingNode
        };
        const tGltfData = await GLTF2Export.GLBAsync(pModel.scene, "scene", tGltfOptions);
        //reset patched textures
        for (const tTexture of tPatchedTextures) {
            tTexture.tex.name = tTexture.orgName;
            tTexture.tex.uOffset = tTexture.orgOffsetU;
            tTexture.tex.vOffset = tTexture.orgOffsetV;
            tTexture.tex.wRotationCenter = 0.5;
            tTexture.tex.uRotationCenter = 0.5;
            tTexture.tex.vRotationCenter = 0.5;
        }
        //remove root scaling node
        if (tScalingNode != null) {
            for (const tElement of tElements) {
                tElement.node.parent = null;
                tElement.updateTransformation();
            }
            tScalingNode.dispose();
        }
        //get glb
        const tSceneGlb = tGltfData.glTFFiles["scene.glb"];
        if (tSceneGlb == null)
            throw newError(ERR_FAILED, "scene.glb not found");
        return tSceneGlb;
    }
}
