/**
 * Converts an ArrayBuffer into a Base64 encoded string.
 */
export declare function toBase64(pBuffer: ArrayBuffer): string;
/**
 * Converts an ArrayBuffer into a hex (Base16) encoded string.
 */
export declare function toHex(pBuffer: ArrayBuffer): string;
/**
 * Converts an ArrayBuffer into a data uri.
 */
export declare function toDataUri(pBuffer: ArrayBuffer, pMimeType: string): string;
/**
 * Converts a Base64 encoded string into an ArrayBuffer.
 */
export declare function fromBase64(pBase64Str: string): ArrayBuffer;
/**
 * Converts a data uri into an ArrayBuffer.
 */
export declare function fromDataUri(pDataUri: string): ArrayBuffer;
/**
 * Converts a Blob into an ArrayBuffer.
 */
export declare function fromBlob(pBlob: Blob): Promise<ArrayBuffer>;
/**
 * Computes the hash for the given buffer and returns it as hex string.
 * If the browser does not support the given algorithm the `jsSHA` library will be used as a fallback.
 * In this case `utils.wcfLibsPath` has to be defined.
 * Note: If chrome is used with a non-secure connection `jsSHA` will be always used.
 */
export declare function hash(pBuffer: ArrayBuffer | Uint8Array, pAlgorithm: "SHA-256" | "SHA-384" | "SHA-512"): Promise<string>;
